/* Nabi - X Input Method server for hangul
 * Copyright (C) 2003 Choe Hwanjin
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <wchar.h>
#include "hangul.h"
#include "server.h"
#include "utfto.h"

NabiIC *
nabi_ic_create(void)
{
	NabiIC * ic;

	ic = (NabiIC *)malloc(sizeof(NabiIC));
	nabi_ic_buf_clear(ic);

	return ic;
}

void
nabi_ic_destroy(NabiIC * ic)
{
	nabi_ic_buf_clear(ic);
}

void
nabi_ic_buf_clear(NabiIC * ic)
{
	int i = 0;
	ic->index = -1;
	for(i = 0; i < 12; i++)
		ic->stack[i] = 0;

	ic->lindex = 0;
	for(i = 0; i < 4; i++)
		ic->choseong[i] = 0;

	ic->vindex = 0;
	for(i = 0; i < 4; i++)
		ic->jungseong[i] = 0;

	ic->tindex = 0;
	for(i = 0; i < 4; i++)
		ic->jongseong[i] = 0;
}

void
nabi_ic_push(NabiIC *ic, wchar_t ch)
{
    ic->stack[++ic->index] = ch;
}

wchar_t
nabi_ic_peek(NabiIC *ic)
{
    if (ic->index < 0)
	return 0;
    return ic->stack[ic->index];
}

wchar_t
nabi_ic_pop(NabiIC *ic)
{
    if (ic->index < 0)
	return 0;
    return ic->stack[ic->index--];
}

void
nabi_ic_get_preedit_string(NabiIC *ic, char *utf8, int buflen,
			   int *len, int *size)
{
    int i, n;
    wchar_t ch;
    wchar_t buf[16];
    
    n = 0;
    if (nabi_server->output_mode == NABI_OUTPUT_MANUAL) {
	/* we use conjoining jamo, U+1100 - U+11FF */
	if (ic->choseong[0] == 0)
	    buf[n++] = HCF;
	else {
	    for (i = 0; i <= ic->lindex; i++)
		buf[n++] = ic->choseong[i];
	}
	if (ic->jungseong[0] == 0)
	    buf[n++] = HJF;
	else {
	    for (i = 0; i <= ic->vindex; i++)
		buf[n++] = ic->jungseong[i];
	}
	if (ic->jongseong[0] != 0) {
	    for (i = 0; i <= ic->tindex; i++)
		buf[n++] = ic->jongseong[i];
	}
    } else if (nabi_server->output_mode == NABI_OUTPUT_JAMO) {
	/* we use conjoining jamo, U+1100 - U+11FF */
	if (ic->choseong[0] == 0)
	    buf[n++] = HCF;
	else
	    buf[n++] = ic->choseong[0];
	if (ic->jungseong[0] == 0)
	    buf[n++] = HJF;
	else
	    buf[n++] = ic->jungseong[0];
	if (ic->jongseong[0] != 0)
	    buf[n++] = ic->jongseong[0];
    } else {
	if (ic->choseong[0]) {
	    if (ic->jungseong[0]) {
		/* have cho, jung, jong or no jong */
		ch = hangul_jamo_to_syllable(ic->choseong[0],
				 ic->jungseong[0],
				 ic->jongseong[0]);
		buf[n++] = ch;
	    } else {
		if (ic->jongseong[0]) {
		    /* have cho, jong */
		    ch = hangul_choseong_to_cjamo(ic->choseong[0]);
		    buf[n++]  = ch;
		    ch = hangul_jongseong_to_cjamo(ic->jongseong[0]);
		    buf[n++] = ch;
		} else {
		    /* have cho */
		    ch = hangul_choseong_to_cjamo(ic->choseong[0]);
		    buf[n++] = ch;
		}
	    }
	} else {
	    if (ic->jungseong[0]) {
		if (ic->jongseong[0]) {
		    /* have jung, jong */
		    ch = hangul_jungseong_to_cjamo(ic->jungseong[0]);
		    buf[n++] = ch;
		    ch = hangul_jongseong_to_cjamo(ic->jongseong[0]);
		    buf[n++] = ch;
		} else {
		    /* have jung */
		    ch = hangul_jungseong_to_cjamo(ic->jungseong[0]);
		    buf[n++] = ch;
		}
	    } else {
		if (ic->jongseong[0]) {
		    /* have jong */
		    ch = hangul_jongseong_to_cjamo(ic->jongseong[0]);
		    buf[n++] = ch;
		} else {
		    /* have nothing */
		    ;
		}
	    }
	}
    }

    buf[n] = L'\0';
    *len = n;
    *size = hangul_wcharstr_to_utf8str(buf, utf8, buflen);
}

void
nabi_ic_preedit_insert(NabiIC *ic)
{
    int len, size;
    char buf[48] = { 0, };

    nabi_ic_get_preedit_string(ic, buf, sizeof(buf), &len, &size);
#if 1
	if(!strcmp(buf, "")) {
		buf[0] = ' '; buf[1] = ' ';
	}
	if(ENCODING_EUCKR) {
		char euc_buf[48] = {0, };
		utf8_to_euckr(buf, euc_buf);
		fprintf(stderr, "[%s]", euc_buf);
	}
	else {
		fprintf(stderr, "[%s]", buf);
	}
#endif
}

void
nabi_ic_preedit_update(NabiIC *ic)
{
    int len, size;
    char buf[48] = { 0, };

    nabi_ic_get_preedit_string(ic, buf, sizeof(buf), &len, &size);
#if 1
	if(!strcmp(buf, "")) {
		buf[0] = ' '; buf[1] = ' ';
	}
	if(ENCODING_EUCKR) {
		char euc_buf[48] = {0, };
		utf8_to_euckr(buf, euc_buf);
		fprintf(stderr, "[%s]", euc_buf);
	}
	else {
		fprintf(stderr, "[%s]", buf);
	}
#endif

    if (len == 0) {
	return;
    }
}

int
nabi_ic_commit(NabiIC *ic)
{
    int i, n;
    wchar_t buf[16];
    char utf8buf[48];
 
    buf[0] = L'\0';

    if (nabi_ic_is_empty(ic))
	return 0;

    n = 0;
    if (nabi_server->output_mode == NABI_OUTPUT_MANUAL) {
	/* we use conjoining jamo, U+1100 - U+11FF */
	if (ic->choseong[0] == 0)
	    buf[n++] = HCF;
	else {
	    for (i = 0; i <= ic->lindex; i++)
		buf[n++] = ic->choseong[i];
	}
	if (ic->jungseong[0] == 0)
	    buf[n++] = HJF;
	else {
	    for (i = 0; i <= ic->vindex; i++)
		buf[n++] = ic->jungseong[i];
	}
	if (ic->jongseong[0] != 0) {
	    for (i = 0; i <= ic->tindex; i++)
		buf[n++] = ic->jongseong[i];
	}
    } else if (nabi_server->output_mode == NABI_OUTPUT_JAMO) {
	/* we use conjoining jamo, U+1100 - U+11FF */
	if (ic->choseong[0] == 0)
	    buf[n++] = HCF;
	else
	    buf[n++] = ic->choseong[0];
	if (ic->jungseong[0] == 0)
	    buf[n++] = HJF;
	else
	    buf[n++] = ic->jungseong[0];
	if (ic->jongseong[0] != 0)
	    buf[n++] = ic->jongseong[0];
    } else {
	/* use hangul syllables (U+AC00 - U+D7AF)
	 * and compatibility jamo (U+3130 - U+318F) */
	wchar_t ch;
	ch = hangul_jamo_to_syllable(ic->choseong[0],
			 ic->jungseong[0],
			 ic->jongseong[0]);
	if (ch)
	    buf[n++] = ch;
	else {
	    if (ic->choseong[0]) {
		ch = hangul_choseong_to_cjamo(ic->choseong[0]);
		buf[n++] = ch;
	    }
	    if (ic->jungseong[0]) {
		ch = hangul_jungseong_to_cjamo(ic->jungseong[0]);
		buf[n++] = ch;
	    }
	    if (ic->jongseong[0]) {
		ch = hangul_jongseong_to_cjamo(ic->jongseong[0]);
		buf[n++] = ch;
	    }
	}
    }
    buf[n] = L'\0';
    nabi_ic_buf_clear(ic);

    hangul_wcharstr_to_utf8str(buf, utf8buf, sizeof(utf8buf));
#if 1
	if(ENCODING_EUCKR) {
		char euc_buf[48] = {0, };
		utf8_to_euckr(utf8buf, euc_buf);
		fprintf(stderr, "%s\n", euc_buf);
	}
	else {
		fprintf(stderr, "%s\n", utf8buf);
	}
#endif
    return 1;
}

int
nabi_ic_commit_unicode(NabiIC *ic, wchar_t ch)
{
    wchar_t buf[16];
    char utf8buf[48];
 
    buf[0] = ch;
    buf[1] = L'\0';

    hangul_wcharstr_to_utf8str(buf, utf8buf, sizeof(utf8buf));
#if 1
	if(ENCODING_EUCKR) {
		char euc_buf[48] = {0, };
		utf8_to_euckr(utf8buf, euc_buf);
		fprintf(stderr, "%s\n", euc_buf);
	}
	else {
		fprintf(stderr, "%s\n", utf8buf);
	}
#endif
    return 1;
}

int
nabi_ic_commit_keyval(NabiIC *ic, wchar_t ch, int keyval)
{
    /* need for sebeol symbol */
    if (ch != 0)
	return nabi_ic_commit_unicode(ic, ch);

    /* ISO10646 charcode keyval */
    if ((keyval & 0xff000000) == 0x01000000)
	return nabi_ic_commit_unicode(ic, keyval & 0x00ffffff);

    if ((keyval >= 0x0020 && keyval <= 0x007e) ||
	(keyval >= 0x00a0 && keyval <= 0x00ff))
	return nabi_ic_commit_unicode(ic, keyval);

    /* Forward all other keys */
    return 0;
}
