/*
 * main.c
 * test program for nabi automata
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>
#include <wchar.h>

#include <termios.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <errno.h>

#include "server.h"
#include "nabi.h"

NabiServer * nabi_server = NULL;

int
get_shift_state(void)
{
	int na = 6, ns = 0;
	if(ioctl(fileno(stdin), TIOCLINUX, &na) == 0) ns = na;
	return ns;
}


int
main(void)
{
	struct termios ts, ots;
	char c;

	NabiIC * nabi_ic;
	int shift_state;
	int input_type = 0;

	fprintf(stderr, "Nabi automata test: Type \'$\' to exit\n");
	fprintf(stderr, "Control-Space for Korean/English conversion\n");

	tcgetattr(STDIN_FILENO, &ts);
	ots = ts;
	ts.c_lflag &= ~ICANON;
	ts.c_lflag &= ~ECHO;
	tcsetattr(STDIN_FILENO, TCSAFLUSH, &ts);

	tcgetattr(STDIN_FILENO, &ts);
	if(ts.c_lflag & ICANON) {
		fprintf(stderr, "Nabi: Failed (un)canonizing\n");
		exit(1);
	}

	nabi_server = nabi_server_new();
	nabi_app_init("2");
	nabi_server_set_output_mode(nabi_server, NABI_OUTPUT_SYLLABLE);
	nabi_server_set_encoding_type(nabi_server, NABI_ENCODING_EUCKR);

	nabi_ic = nabi_ic_create();

	while(1) {
		c = getchar();
		if(c == '$') break;
		if(c == 0) {
			if(input_type == 0) input_type = 1;
			else input_type = 0;
			continue;
		}
		shift_state = get_shift_state();

		if(input_type == 0) {
			fprintf(stderr, "%c\n", c);
		}
		else if(!nabi_server->automata(nabi_ic, c, shift_state)) {
			fprintf(stderr, "%c\n", c);
		}
	}

	tcsetattr(STDIN_FILENO, TCSANOW, &ots);

	nabi_ic_destroy(nabi_ic);
	nabi_server_destroy(nabi_server);
	fprintf(stderr, "\nNabi: Exited safely\n");

	return 0;
}
