/* Nabi - X Input Method server for hangul
 * Copyright (C) 2003 Choe Hwanjin
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif


#include <stdio.h>
#include <stdlib.h>
#include <wchar.h>
#include "server.h"
#include "nabi.h"

int nabi_automata_2(NabiIC * ic, int keyval, unsigned int state);
int nabi_automata_3(NabiIC * ic, int keyval, unsigned int state);

NabiServer *
nabi_server_new(void)
{
	NabiServer * server;
	server = (NabiServer *)malloc(sizeof(NabiServer));

	server->keyboard_map = NULL;
	server->compose_map = NULL;
	server->automata = NULL;
	server->dvorak = 0;
	server->encoding_type = NABI_ENCODING_UTF8;

	return server;
}

void
nabi_server_destroy(NabiServer * server)
{
	int i;

	if(server == NULL)
		return;

	for(i = 0; i < server->compose_map_size; i++)
		if(server->compose_map[i] != NULL)
			free(server->compose_map[i]);

	if(&(server->keyboard_map) != NULL)
		free(&(server->keyboard_map));

	free(server);
}

void
nabi_server_set_keyboard(NabiServer * server,
				const wchar_t * keyboard_map,
				NabiKeyboardType type)
{
	if(server == NULL)
		return;

	server->keyboard_map = keyboard_map;

	if(type == NABI_KEYBOARD_2SET)
		server->automata = nabi_automata_2;
	else
		server->automata = nabi_automata_3;
}

void
nabi_server_set_compose_map(NabiServer * server,
				NabiComposeItem ** compose_map,
				int size)
{
	if(server == NULL)
		return;

	server->compose_map = compose_map;
	server->compose_map_size = size;
}

void
nabi_server_set_automata(NabiServer * server, NabiKeyboardType type)
{
	if(server == NULL)
		return;

	if(type == NABI_KEYBOARD_2SET)
		server->automata = nabi_automata_2;
	else
		server->automata = nabi_automata_3;
}

void
nabi_server_set_dvorak(NabiServer * server, int flag)
{
	if(server == NULL)
		return;

	server->dvorak = flag;
}

void
nabi_server_set_output_mode(NabiServer * server, NabiOutputMode mode)
{
	if(server == NULL)
		return;

	server->output_mode = mode;
}

void
nabi_server_set_encoding_type(NabiServer * server, NabiEncodingType type)
{
	if(server == NULL)
		return;

	server->encoding_type = type;
}
