/* Nabi - X Input Method server for hangul
 * Copyright (C) 2003 Choe Hwanjin
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include <stdint.h> 
#include "ic.h"

typedef struct _NabiComposeItem NabiComposeItem;
typedef struct _NabiServer NabiServer;

struct _NabiComposeItem {
	uint32_t key;
	wchar_t code;
};

typedef enum {
	NABI_OUTPUT_SYLLABLE,
	NABI_OUTPUT_JAMO,
	NABI_OUTPUT_MANUAL
} NabiOutputMode;

typedef enum {
	NABI_KEYBOARD_2SET,
	NABI_KEYBOARD_3SET
} NabiKeyboardType;

typedef enum {
	NABI_ENCODING_UTF8,
	NABI_ENCODING_EUCKR
} NabiEncodingType;

struct _NabiServer {
	/* hangul automata */
	const wchar_t *	keyboard_map;
	NabiComposeItem ** compose_map;
	int compose_map_size;
	int (*automata)(NabiIC *, int, unsigned int);
	int dvorak;
	NabiOutputMode output_mode;
	NabiEncodingType encoding_type;
};

extern NabiServer * nabi_server;

NabiServer * nabi_server_new(void);
void nabi_server_destroy(NabiServer * server);
void nabi_server_set_keyboard(NabiServer * server,
				const wchar_t * keyboard_map,
				NabiKeyboardType type);
void nabi_server_set_compose_map(NabiServer * server, 
				NabiComposeItem ** compose_map,
				int size);
void nabi_setver_set_automata(NabiServer * server,
				NabiKeyboardType type);
void nabi_server_set_dvorak(NabiServer * server,
				int flag);
void nabi_server_set_output_mode(NabiServer * server,
				NabiOutputMode mode);
void nabi_server_set_encoding_type(NabiServer * server,
				NabiEncodingType encoding_type);
